TRANSACTION_MIN_GAS_PRICE = 1000000000
TRANSACTION_VERSION_DEFAULT = 2
TRANSACTION_OPTIONS_DEFAULT = 0
ARGS_SEPARATOR = "@"
METACHAIN_ID = 4294967295
VM_TYPE_WASM_VM = bytes([0x05, 0x00])

# 64 bytes = 512 bits
INTEGER_MAX_NUM_BYTES = 64

REWA_TOKEN_IDENTIFIER = "REWA"
REWA_NUM_DECIMALS = 18

DELEGATION_MANAGER_SC_ADDRESS = "moa1qqqqqqqqqqqqqqqpqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqylllsjrx4c2"
DEFAULT_HRP = "moa"
CONTRACT_DEPLOY_ADDRESS = "moa1qqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqqhsx6tv"
TRANSACTION_OPTIONS_TX_GUARDED = 0b0010

DIGEST_SIZE = 32

TOKEN_RANDOM_SEQUENCE_LENGTH = 6
